#!/usr/bin/env python3

import timedata, timeit

SIZE = 1024

CLASSIC = [0] * (3 * SIZE)
TRIPLES = [(0, 0, 0)] * SIZE

TIMEDATA = timedata.OperatorTest()
TIMEDATA.resize(SIZE)

def operator_add(arg):
    TIMEDATA.operator_add(2)

def class_add(arg):
    TIMEDATA.class_add(2)

def operator_pow(arg):
    TIMEDATA.operator_pow(2)

def class_pow(arg):
    TIMEDATA.class_pow(2)

def run(command, arg, number=1000):
    return timeit.Timer(lambda: command(arg)).timeit(number=number)

print('tests')

for arg in 2, TIMEDATA:
    print('-->', 'number' if arg == 2 else 'list', '\n')
    for t in 'add', 'pow':
        for s in 'operator', 'class':
            print('%8s %s' % (s, t), 100 * run(locals()['_'.join((s, t))], arg))



import unittest

from timedata import Color, Colors, ColorList

class TestColorList(unittest.TestCase):
    def test_trivial(self):
        cl = ColorList()
        self.assertEqual(len(cl), 0)
        cl.reverse()
        self.assertEqual(len(cl), 0)

    def test_construct(self):
        cl = ColorList([[1, 1, 1]])
        self.assertEqual(str(cl), str(ColorList(['white'])))
        self.assertEqual(cl, ColorList(['white']))

    def test_arithmetic(self):
        cl = ColorList(('red', 'green', 'blue'))
        self.assertEqual(cl.copy().add(cl), cl.copy().mul(2))
        # self.assertEqual(2 + (2 * cl * 3) + 2 - 2 * cl, 4 + 4 * cl)

    def test_slicing(self):
        cl = ColorList()
        cl[:] = 'red', 'green', 'blue'
        self.assertEqual(cl, ColorList(['red', 'green', 'blue']))
        self.assertEqual(cl[1:2], ColorList(['green']))

    def test_rotate(self):
        cl = ColorList(('red', 'green', 'blue'))
        cl.rotate(1)
        self.assertEqual(cl, ColorList(['green', 'blue', 'red']))

    def test_append(self):
        cl = ColorList()
        cl.append('red')
        self.assertEqual(cl, ColorList(('red')))

        cl.append('green')
        self.assertEqual(cl, ColorList(('red', 'green')))

        with self.assertRaises(ValueError):
            cl.append('wombat')
        self.assertEqual(cl, ColorList(('red', 'green')))

    def test_clear(self):
        self.assertEqual(ColorList(('red', 'green')).clear(),
                         ColorList())
        self.assertEqual(ColorList(('red', 'green')).zero(),
                         ColorList(('black', 'black')))

    def test_mult(self):
        cl = ColorList(('red', 'green', 'blue'))
        cl2 = ColorList(('red', 'green', 'blue', 'red', 'green', 'blue',
                        'red', 'green', 'blue'))
        self.assertEqual(cl * 3, cl2)
        self.assertEqual(3 * cl, cl2)
        self.assertEqual(cl * 1, cl)
        self.assertEqual(1 * cl, cl)
        self.assertEqual(cl * 0, ColorList())
        self.assertEqual(0 * cl, ColorList())

        cl *= 3
        self.assertEqual(cl, cl2)
        cl *= 1
        self.assertEqual(cl, cl2)
        cl *= 0
        self.assertEqual(cl, ColorList())

        cl = ColorList(('red', 'green', 'blue'))
        with self.assertRaises(OverflowError):
            cl *= -1
        with self.assertRaises(OverflowError):
            cl * -1
        self.assertEqual(cl, ColorList(('red', 'green', 'blue')))

    def test_add(self):
        cl = ColorList(['red', 'green', 'blue'])
        cl += ColorList(['yellow', 'black'])
        self.assertEqual(cl, ColorList(
            ['red', 'green', 'blue', 'yellow', 'black']))
        self.assertEqual(cl, cl + ColorList())
        self.assertEqual(cl, ColorList() + cl)

    def test_extend(self):
        cl = ColorList()
        cl.extend(('red', 'green', 'blue'))
        self.assertEqual(cl, ColorList(['red', 'green', 'blue']))
        try:
            cl.extend(1)
        except:
            pass
        else:
            self.assertTrue(False)
        self.assertEqual(cl, ColorList(['red', 'green', 'blue']))

    def test_indexing(self):
        red, green, blue = (
            Color('red'), Color('green'), Color('blue'))
        c = ColorList((red, green, blue))
        self.assertEqual(c[0], red)
        self.assertEqual(c[1], green)
        self.assertEqual(c[2], blue)
        self.assertEqual(c[-3], red)
        self.assertEqual(c[-2], green)
        self.assertEqual(c[-1], blue)
        with self.assertRaises(IndexError):
            c[-4]
        with self.assertRaises(IndexError):
            c[3]

        r, g, b = c
        self.assertEqual(r, red)
        self.assertEqual(g, green)
        self.assertEqual(b, blue)

    def test_append(self):
        cl = ColorList()
        cl.append(Colors.red)
        cl.append(Color(1, 1, 1))
        self.assertEqual(cl, ColorList(('red', 'white')))

    def test_div_pow(self):
        c = ColorList(('red', 'green', 'blue'))
        c[1] = -c[1]
        cl = c.copy().div(0)
        self.assertEqual(cl[0][0], float('inf'))
        self.assertFalse(cl[0][1] == cl[0][1])
        self.assertEqual(cl[1][1], float('-inf'))

        self.assertEqual(c.rpow(0), ColorList(('cyan', 'magenta', 'yellow')))

    def test_max_min(self):
        self.assertEqual(ColorList().max(), Colors.black)
        self.assertEqual(ColorList().min(), Colors.black)

        c = ColorList(('red', 'green', 'blue'))
        self.assertEqual(c.max(), Colors.white)
        self.assertEqual(c.min(), Colors.black)

        c.min_limit(0.25).max_limit(0.75)

        gray = Colors.white * 0.25
        self.assertEqual(c, ColorList((gray + Colors.red / 2,
                                       gray + Colors.green / 2,
                                       gray + Colors.blue / 2)))

    def test_round(self):
        cl = ColorList(((0.15, 0.24, 0.36), (-0.15, -0.24, -0.36)))
        self.assertEqual(cl.round(1),
                             ColorList(((0.2, 0.2, 0.4), (-0.2, -0.2, -0.4))))

    def test_distance(self):
        cl = ColorList()
        self.assertEqual(cl.distance(ColorList()), 0)
        self.assertEqual(cl.distance2(ColorList()), 0)

        self.assertEqual(cl.distance(ColorList(['black'])), 0)
        self.assertEqual(cl.distance2(ColorList(['black'])), 0)

        self.assertEqual(cl.distance2(ColorList(['white'])), 3)

    def test_list_ops(self):
        cl = ColorList(('red', 'green', 'blue'))
        cl2 = cl.copy()
        self.assertEqual(cl, cl2)
        self.assertIsNot(cl, cl2)
        self.assertEqual(cl.count(Colors.red), 1)
        self.assertEqual(cl.count(Colors.yellow), 0)
        self.assertEqual(cl.index(Colors.blue), 2)
        with self.assertRaises(ValueError):
            cl.index(Colors.white)

    def test_insert(self):
        cl = ColorList(('red', 'green', 'blue'))
        self.assertEqual(cl.copy().insert(0, Colors.white),
                         ColorList(('white', 'red', 'green', 'blue')))
        self.assertEqual(cl.copy().insert(1, Colors.white),
                         ColorList(('red', 'white', 'green', 'blue')))
        self.assertEqual(cl.copy().insert(2, Colors.white),
                         ColorList(('red', 'green', 'white', 'blue')))
        self.assertEqual(cl.copy().insert(3, Colors.white),
                         ColorList(('red', 'green', 'blue', 'white')))

        self.assertEqual(cl.copy().insert(-1, Colors.white),
                         ColorList(('red', 'green', 'white', 'blue')))
        self.assertEqual(cl.copy().insert(-2, Colors.white),
                         ColorList(('red', 'white', 'green', 'blue')))
        self.assertEqual(cl.copy().insert(-3, Colors.white),
                         ColorList(('white', 'red', 'green', 'blue')))

        self.assertEqual(cl.copy().insert(-4, Colors.white),
                         ColorList(('white', 'red', 'green', 'blue')))
        self.assertEqual(cl.copy().insert(-5, Colors.white),
                         ColorList(('white', 'red', 'green', 'blue')))

    def test_pop(self):
        cl = ColorList(('red', 'green', 'blue'))
        cl2 = cl.copy()
        self.assertEqual(cl2.pop(), Colors.blue)
        self.assertEqual(cl2, cl[:2])

        cl2 = cl.copy()
        self.assertEqual(cl2.pop(-1), Colors.blue)
        self.assertEqual(cl2, cl[:2])

        cl2 = cl.copy()
        self.assertEqual(cl2.pop(0), Colors.red)
        self.assertEqual(cl2, cl[1:])

        cl2 = cl.copy()
        self.assertEqual(cl2.pop(-3), Colors.red)
        self.assertEqual(cl2, cl[1:])

        cl2 = cl.copy()
        self.assertEqual(cl2.pop(1), Colors.green)
        self.assertEqual(cl2, cl[0:3:2])

        cl2 = cl.copy()
        self.assertEqual(cl2.pop(-2), Colors.green)
        self.assertEqual(cl2, cl[0:3:2])

        cl2 = cl.copy()
        self.assertEqual(cl2.pop(2), Colors.blue)
        self.assertEqual(cl2, cl[:2])

        with self.assertRaises(IndexError):
            cl.copy().pop(3)
        with self.assertRaises(IndexError):
            cl.copy().pop(4)
        with self.assertRaises(IndexError):
            cl.copy().pop(-4)
        with self.assertRaises(IndexError):
            cl.copy().pop(-5)

    def test_remove(self):
        cl = ColorList(('green', 'blue', 'red'))
        cl2 = cl.copy()
        self.assertEqual(cl2.remove(Colors.red), cl[:-1])
        cl2 = cl.copy()
        self.assertEqual(cl2.remove(Colors.green), cl[1:])
        with self.assertRaises(ValueError):
            cl.copy().remove(Colors.white)

    def test_sort(self):
        cl = ColorList(('red', 'green', 'blue'))
        self.assertEqual(cl.copy().sort(reverse=True), cl)
        self.assertEqual(cl.copy().sort(), cl[::-1])

    def test_str(self):
        cl = ColorList(((-0.2, -0.2, -0.4), (0.2, 0.2, 0.4)))
        self.assertEqual(str(cl),
                         '((-0.2, -0.2, -0.4), (0.2, 0.2, 0.4))')



#!/usr/bin/env python3

import timedata, timeit

SIZE = 10240

CLASSIC = [0] * (3 * SIZE)
TRIPLES = [(0, 0, 0)] * SIZE

def make_list(maker):
    m = maker()
    m.resize(SIZE)
    return m

TIMEDATA = make_list(timedata.ColorList)

def scale_functional(colors):
    return [c * 2 for c in colors]

def scale_classic(colors):
    for i in range(len(colors)):
        colors[i] *= 2

def scale_triples(colors):
    for i, c in enumerate(colors):
        r, g, b = c
        colors[i] = (2 * r, 2 * g, 2 * b)

def scale_timedata(colors):
    colors *= 2

def scale_timedata(colors):
    colors *= 2

def scale_new(colors):
    colors = colors * 2

def run(command, value, number=200):
    return 100 * timeit.Timer(lambda: command(value)).timeit(number=number)

print('scale\n')
print('func:     ', run(scale_functional, CLASSIC))
print('classic:  ', run(scale_classic, CLASSIC))
print('triples:  ', run(scale_triples, TRIPLES))
print('timedata:     ', run(scale_timedata, TIMEDATA))
print('timedata2:    ', run(scale_new, TIMEDATA))


def gamma_classic(colors):
    for i in range(len(colors)):
        colors[i] **= 1.2

def gamma_triples(colors):
    for i, c in enumerate(colors):
        r, g, b = c
        colors[i] = (r ** 1.2, g ** 1.2, g ** 1.2)

def gamma_timedata(colors):
    colors **= 1.2

def gamma_new(colors):
    colors = colors ** 1.2

# From https://github.com/scottjgibson/PixelPi/blob/master/pixelpi.py
LPD8806 = [int(pow(float(i) / 255.0, 2.5) * 255.0 + 0.5) for i in range(256)]
APA102 = LPD8806
WS2801 = [int(pow(float(i) / 255.0, 2.5) * 255.0) for i in range(256)]
SM16716 = [int(pow(float(i) / 255.0, 2.5) * 255.0) for i in range(256)]

# From http://rgb-123.com/ws2812-color-output/
WS2812B = NEOPIXEL = WS2812 = [int(pow(float(i) / 255.0, 1.0 / 0.45) * 255.0) for i in range(256)]

def gamma_classic_table(colors):
    for i in range(len(colors)):
        colors[i] = LPD8806[colors[i]]


print('\n\ngamma\n')
print('classic:      ', run(gamma_classic, CLASSIC))

CLASSIC = [int(c) for c in CLASSIC] # clear it out!
print('classic table:', run(gamma_classic_table, CLASSIC))
print('triples:      ', run(gamma_triples, TRIPLES))
print('timedata:         ', run(gamma_timedata, TIMEDATA))
print('timedata new:     ', run(gamma_new, TIMEDATA))


# From https://stackoverflow.com/questions/449560

import sys
from numbers import Number
from collections import Set, Mapping, deque

try: # Python 2
    zero_depth_bases = (basestring, Number, xrange, bytearray)
    iteritems = 'iteritems'
except NameError: # Python 3
    zero_depth_bases = (str, bytes, Number, range, bytearray)
    iteritems = 'items'

def getsize(obj):
    """Recursively iterate to sum size of object & members."""
    def inner(obj, _seen_ids = set()):
        obj_id = id(obj)
        if obj_id in _seen_ids:
            return 0
        _seen_ids.add(obj_id)
        size = sys.getsizeof(obj)
        if isinstance(obj, zero_depth_bases):
            pass # bypass remaining control flow and return
        elif isinstance(obj, (tuple, list, Set, deque)):
            size += sum(inner(i) for i in obj)
        elif isinstance(obj, Mapping) or hasattr(obj, iteritems):
            size += sum(inner(k) + inner(v) for k, v in getattr(obj, iteritems)())
        # Now assume custom object instances
        elif hasattr(obj, '__slots__'):
            size += sum(inner(getattr(obj, s)) for s in obj.__slots__ if hasattr(obj, s))
        else:
            attr = getattr(obj, '__dict__', None)
            if attr is not None:
                size += inner(attr)
        return size
    return inner(obj)


print('\n\nsizes\n')
print('classic:', getsize(CLASSIC))
print('triples:', getsize(TRIPLES))
print('timedata:   ', getsize(TIMEDATA))



import unittest

from timedata import compile_timestamp, Colors, Colors256, Color, Color256

class TestGeneral(unittest.TestCase):
    def test_timestamp(self):
        self.assertEqual(len(compile_timestamp()), 26)
        self.assertEqual(compile_timestamp()[:3], '201')

class TestColors(unittest.TestCase):
    def test_all(self):
        self.assertEqual(Colors.red, Color('red'))
        self.assertEqual(Colors256.cyan, Color256('cyan'))



import abc, timedata

import itertools

class Driver(object):
    __metaclass__ = abc.ABCMeta

    PERMUTATIONS = tuple(itertools.permutations('rgb'))

    # The list of parameter that you're setting on
    PARAMS = 'brightness', 'gamma', 'max', 'min', 'permutation'

    BRIGHTNESS = 1.0
    GAMMA = 1.0
    MAX = 255
    MIN = 0
    PERMUTATION = 0
    RENDERER = timedata.Renderer

    def __init__(self, size, renderer=self.RENDERER, **kwds):
        unknown = set(kwds) - set(self.PARAMS)
        if unknown:
            raise ValueError('Unknown keywords: ' + ''.join(unknown))

        self.size = size
        self.header = bytearray(self.header(), encoding='ascii')
        self.renderer = renderer(size=size, offset=len(self.header))
        self.footer = bytearray(self.footer(), encoding='ascii')
        self.message = (
            self.header + bytearray(self.renderer.byte_size) + self.footer)
        self.renderer.set_message(self.message)

        for p in self.PARAMS:
            setattr(self.renderer, p, kwds.get(p, getattr(self, p.upper())))

        # If we don't have a render method, use the one from the renderer.
        self.render = getattr(self, 'render', self.renderer)

    def _body_size(self):
        # Override for non-RGB, etc.
        return 3 * self._size

    @abc.abstractmethod
    def header(self):
        """Return a string with the header of the message, before the lights."""
        return

    @abc.abstractmethod
    def footer(self):
        """Return a string with the footer of the message, after the lights."""
        return



import unittest

from timedata import Stripe, Combiner, Render3

class TestGenerated(unittest.TestCase):
    def test_stripe(self):
        s = Stripe()
        self.assertEqual(str(s), '(begin=0, skip=1, repeats=1, reflect=False)')
        self.assertEqual(
            repr(s), 'timedata.Stripe(begin=0, skip=1, repeats=1, reflect=False)')

        s.skip = 2
        s.reflect = True
        self.assertEqual(str(s), '(begin=0, skip=2, repeats=1, reflect=True)')

    def test_render(self):
        s = Render3()
        self.assertEqual(str(s), '(min=0.0, max=255.0, scale=1.0, gamma=1.0, '
                         'permutation=\'rgb\', offset=0, size=0)')



import unittest

from timedata import *

class TestOperations(unittest.TestCase):
    def assertSpread(self, args, expect):
        expected = ColorList(expect)
        actual = ColorList.spread(*args)
        failures = []
        if len(expected) != len(actual):
            failures.append('size: %s %s' % (len(expected), len(actual)))
        for i, (e, a) in enumerate(zip(expected, actual)):
            if e.distance2(a) > 0.0001:
                failures.append('color: %s %s %s' % (i, e, a))
        if failures:
            msg = '\n'.join(['Failure', str(args), str(expect), str(actual)]
                             + failures)
            self.fail(msg)


    def test_trivial_spread(self):
        self.assertSpread([], [])
        self.assertSpread(['white'], ['white'])
        self.assertSpread(['white', 'black'], ['white', 'black'])
        self.assertSpread(['white', 0, 'black'], ['white', 'black'])
        self.assertSpread(['white', 1, 'black'], ['white', 'gray 50', 'black'])
        self.assertSpread(['white', 3, 'black'],
                          ['white', 'gray 75', 'gray 50', 'gray 25', 'black'])

    def test_spread(self):
        r, g = Colors.red, Colors.green
        self.assertSpread(
            [r, 2, 6, g],
            [r,
             r + 1 * (g - r) / 9,
             r + 2 * (g - r) / 9,
             r + 3 * (g - r) / 9,
             r + 4 * (g - r) / 9,
             r + 5 * (g - r) / 9,
             r + 6 * (g - r) / 9,
             r + 7 * (g - r) / 9,
             r + 8 * (g - r) / 9,
             g])



import unittest

from timedata import Color, Color256, Colors, Colors256, NewColor255, NewColor256, NewColor

Color = NewColor
Color256 = NewColor256

class TestColor(unittest.TestCase):
    def test_white(self):
        white = Color(1, 1, 1)
        self.assertEqual(str(white), 'white')
        self.assertEqual(white, Color((1, 1, 1)))
        self.assertEqual(white[:], white)
        self.assertEqual(white[::-1], white)
        self.assertEqual(white[0:2], (1.0, 1.0))

    def test_red(self):
        red = Color(1, 0, 0)
        blue = Color('blue')

        self.assertEqual(str(red), 'red')
        self.assertEqual(red[:], red)
        self.assertEqual(red[::-1], blue)
        self.assertEqual(red[-3:3:1], red)
        self.assertEqual(red[-1:-3:-1], (0.0, 0.0))
        self.assertEqual(red[-3:-1], (1.0, 0.0))
        self.assertEqual(red[0:2], (1.0, 0.0))

    def test_mixed(self):
        antired = Color(-1, 0, 0)
        self.assertEqual(str(antired), 'red-++')
        self.assertEqual(antired + Color('red'), Color('black'))
        self.assertEqual(str(-Color('white')), 'white---')

    def test_arithmetic(self):
        black, red, green, blue, white, yellow, cyan, magenta = (
            Color(i) for i in ('black', 'red', 'green', 'blue', 'white',
                               'yellow', 'cyan', 'magenta'))

        self.assertEqual(red + green + blue, white)
        self.assertEqual(red + green, yellow)
        self.assertEqual(green + blue, cyan)
        self.assertEqual(red + blue, magenta)

        self.assertEqual(white - blue, yellow)
        self.assertEqual(white - green, magenta)
        self.assertEqual(white - red, cyan)

        self.assertEqual(~white, black)
        self.assertEqual(~black, white)

        self.assertEqual(white * red, red)
        self.assertEqual(white * cyan, cyan)

    def test_methods(self):
        self.assertEqual(Color().start, 0)
        self.assertEqual(Color().range, 1.0)

    def test_rotated(self):
        self.assertEqual(Color('red').rotated(1), Color('blue'))
        g1 = (Color('red') + Color('blue') * 0.5).rotated(-1)
        g2 = Color('green') + Color('red') * 0.5
        # TODO:  but these should be *exactly equal!  Why do I need that?
        self.assertLessEqual(g1.distance(g2), 1.0E-6)

    def test_compare(self):
        for x in 0, -1, 'red', (0.1, 0.1, 0.1):
            x = Color(x)
            self.assertTrue(x == x)
            self.assertFalse(x != x)
            self.assertTrue(x <= x)
            self.assertFalse(x < x)
            self.assertTrue(x >= x)
            self.assertFalse(x > x)

        black, red, green, blue, white, yellow, cyan, magenta = (
            Color(i) for i in ('black', 'red', 'green', 'blue', 'white',
                               'yellow', 'cyan', 'magenta'))
        for x, y in ((red, green), (green, blue), (red + green, red + blue),
                     (white, red), (white, green), (white, blue)):
            self.assertFalse(x == y)
            self.assertTrue(x != y)
            self.assertFalse(x <= y)
            self.assertFalse(x < y)
            self.assertTrue(x >= y)
            self.assertTrue(x > y)

    def test_indexing(self):
        c = Color(1, 2, 3)
        self.assertEqual(c[0], 1)
        self.assertEqual(c[1], 2)
        self.assertEqual(c[2], 3)
        self.assertEqual(c[-3], 1)
        self.assertEqual(c[-2], 2)
        self.assertEqual(c[-1], 3)
        with self.assertRaises(IndexError):
            c[-4]
        with self.assertRaises(IndexError):
            c[3]

        r, g, b = c
        self.assertEqual(c, Color(r, g, b))
        self.assertEqual(c, Color(c))
        self.assertEqual(c, Color(*c))

    def test_abs(self):
        red, green, gray = (
            Color('red'), Color('green'), Color('gray'))
        self.assertEqual(abs(-red), red)
        self.assertEqual(abs(-(red + gray + gray)), red + gray + gray)

    def test_hex(self):
        for k, v in Colors.__dict__.items():
            if not (k.startswith('_') or k.startswith('grey') or
                    k.startswith('gray')):
                h_value = v.to_hex()
                self.assertEqual(Color.from_hex(h_value), Color(v))

    def test_distance(self):
        self.assertEqual(Colors.red.distance2(Colors.green), 2)
        self.assertEqual(Colors.red.distance(Colors.black), 1)

    def DONT_test_everything(self):
        # This test takes about five minutes to run when it's enabled.
        for i in range(256 * 256 * 256):
            if not i % (256 * 256 * 4):
                print('!', end='', flush=True)
            c = Color.from_hex(i)
            self.assertEqual(i, c.to_hex(), i)
            s = str(c)
            c2 = Color(s)
            distance = c.distance(c2)
            self.assertTrue(distance < 0.000001, i)
            self.assertEqual(str(c2), s, i)


class TestColor256(unittest.TestCase):
    def test_first(self):
        white = Color256(256, 256, 256)
        self.assertEqual(str(white), 'white')
        self.assertEqual(white[:], white)
        self.assertEqual(white[::-1], white)
        self.assertEqual(white[0:2], (256, 256))

    def test_string(self):
        self.assertEqual(list(Color256('white')), [256, 256, 256])

    def test_red(self):
        red = Color256((256, 0, 0))
        blue = Color256('blue')

        self.assertEqual(str(red), 'red')
        self.assertEqual(red[:], red)
        self.assertEqual(red[-1:-3:-1], (0.0, 0.0))
        self.assertEqual(red[-3:-1], (256, 0.0))
        self.assertEqual(red[0:2], (256, 0.0))
        self.assertEqual(red[-3:3:1], red)
        self.assertEqual(red[::-1], blue)

    def test_arithmetic(self):
        black, red, green, blue, white, yellow, cyan, magenta = (
            Color256(i) for i in ('black', 'red', 'green', 'blue', 'white',
                                  'yellow', 'cyan', 'magenta'))

        self.assertEqual(red + green + blue, white)
        self.assertEqual(red + green, yellow)
        self.assertEqual(green + blue, cyan)
        self.assertEqual(red + blue, magenta)

        self.assertEqual(white - blue, yellow)
        self.assertEqual(white - green, magenta)
        self.assertEqual(white - red, cyan)

        self.assertEqual(~white, black)
        self.assertEqual(~black, white)

        self.assertEqual(white * red / 256, red)
        self.assertEqual(white * cyan / 256, cyan)

    def test_methods(self):
        a = (Color256('red') + Color256('blue') * 0.5).rotated(-1)
        b = Color256('green') + Color256('red') * 0.5
        self.assertTrue(a.distance(b) < 0.0001)
        self.assertEqual(Color256('red').rotated(1), Color256('blue'))
        self.assertEqual(Color256().range, 256.0)

    def test_names(self):
        names = Color.names()
        self.assertEqual(len(names), 481)
        self.assertEqual(names[:10], [
            'alice blue',
            'antique white',
            'antique white 1',
            'antique white 2',
            'antique white 3',
            'antique white 4',
            'aqua',
            'aquamarine',
            'aquamarine 1',
            'aquamarine 2'])

    def test_limit(self):
        self.assertEqual((Colors.red * 2).max_limit(0), Colors.black)
        self.assertEqual((Colors.red * 2).min_limit(0), Colors.red * 2)
        self.assertEqual((Colors.red * 2).max_limit(1), Colors.red)
        self.assertEqual(tuple((Colors.red * 2).min_limit(1)), (2, 1, 1))
        self.assertEqual(Color(1, 2, 3).limit_max(Color(3, 2, 1)),
                         Color(1, 2, 1))
        self.assertEqual(Color(1, 2, 3).limit_min(Color(3, 2, 1)),
                         Color(3, 2, 3))
